from unittest import mock

from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

from ai_django_core.admin.model_admins.mixins import FetchParentObjectInlineMixin
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import MySingleSignalModel, ForeignKeyRelatedModel


class ForeignKeyRelatedModelTabularInline(FetchParentObjectInlineMixin, admin.TabularInline):
    model = ForeignKeyRelatedModel


class TestFetchParentObjectInlineMixinAdmin(admin.ModelAdmin):
    inlines = (ForeignKeyRelatedModelTabularInline,)


class MockResolverResponse:
    kwargs = None


class FetchParentObjectInlineMixinTest(RequestProviderMixin, TestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.super_user = User.objects.create(username='super_user', is_superuser=True)

        admin.site.register(MySingleSignalModel, TestFetchParentObjectInlineMixinAdmin)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        admin.site.unregister(MySingleSignalModel)

    def test_parent_model_is_set(self):
        obj = MySingleSignalModel.objects.create(value=1)
        model_admin = TestFetchParentObjectInlineMixinAdmin(model=MySingleSignalModel, admin_site=admin.site)

        request = self.get_request(self.super_user)
        inline_list = model_admin.inlines

        self.assertGreater(len(inline_list), 0)

        inline = inline_list[0](parent_model=MySingleSignalModel, admin_site=admin.site)

        return_obj = MockResolverResponse()
        return_obj.kwargs = {'object_id': obj.id}
        with mock.patch.object(model_admin.inlines[0], '_resolve_url', return_value=return_obj):
            inline.get_formset(request=request, obj=obj)

        self.assertEqual(inline.parent_object, obj)



from django.contrib.auth.models import User, AbstractUser
from django.test import TestCase
from rest_framework import status
from rest_framework.reverse import reverse

from ai_django_core.drf.tests import BaseViewSetTestMixin
from testapp.api import views
from testapp.models import MySingleSignalModel


class BaseApiTest(BaseViewSetTestMixin, TestCase):
    def get_default_api_user(self) -> AbstractUser:
        return User.objects.create(username='my-username', is_active=True)


class MySingleSignalModelApiViewTest(BaseApiTest):
    view_class = views.MySingleSignalModelViewSet

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        # Create list of objects
        cls.object_list = [
            MySingleSignalModel.objects.create(value=1),
            MySingleSignalModel.objects.create(value=2),
        ]

    def test_action_not_activated(self):
        with self.assertRaises(AttributeError):
            self.execute_request(
                method='post',
                url=reverse('my-single-signal-model-list'),
                viewset_kwargs={'post': 'create'},
                user=self.default_api_user)

    def test_list_authentication_required(self):
        self.validate_authentication_required(
            url=reverse('my-single-signal-model-list'),
            method='get',
            view='list')

    def test_list_regular(self):
        response = self.execute_request(
            method='get',
            url=reverse('my-single-signal-model-list'),
            viewset_kwargs={'get': 'list'},
            user=self.default_api_user)

        self.assertEqual(response.status_code, status.HTTP_200_OK)
        self.assertEqual(len(response.data), len(self.object_list))



import pytest

from ai_django_core.utils.file import crc, md5_checksum


@pytest.fixture
def gen_test_file(tmp_path):
    def inner(content):
        test_file = tmp_path / 'test_file.txt'
        test_file.write_text(content)
        return test_file
    return inner


@pytest.mark.parametrize(
    'test_func', [
        crc,
        md5_checksum
    ]
)
def test_closes_file(mocker, test_func):
    """
    Tests if the CRC and MD5 checksum functions use a context manager to open the file, to guarantee that the opened
    file descriptor is closed.
    """
    open_mock = mocker.patch('ai_django_core.utils.file.open')
    open_mock.return_value.__enter__.return_value.read.return_value = None  # to make f.read() return None.
    file_mock = mocker.Mock()
    test_func(file_mock)
    assert open_mock.call_args[0][0] == file_mock
    assert open_mock.return_value.__exit__.call_count == 1


@pytest.mark.parametrize(
    'content, crc_result, md5_result', [
        ('The answer to life, the universe, and everything.\n', '0988D2CD', '4efb6393969f501be5c1ba7571f0c09f'),
        ('The quick brown fox jumps over the lazy dog', '414FA339', '9e107d9d372bb6826bd81d3542a419d6'),
        ('', '00000000', 'd41d8cd98f00b204e9800998ecf8427e')
    ]
)
def test_crc_and_md5(gen_test_file, content, crc_result, md5_result):
    """Generates the CRC and MD5 of a test file and checks its result"""
    test_file = str(gen_test_file(content))
    result = crc(test_file)
    assert result == crc_result
    result = md5_checksum(test_file)
    assert result == md5_result



from django.core.mail import EmailMultiAlternatives
from django.test import TestCase
from django.test import override_settings

from ai_django_core.mail.backends.whitelist_smtp import WhitelistEmailBackend


@override_settings(EMAIL_BACKEND='ai_django_core.mail.backends.whitelist_smtp.WhitelistEmailBackend',
                   EMAIL_BACKEND_DOMAIN_WHITELIST=['valid.domain'],
                   EMAIL_BACKEND_REDIRECT_ADDRESS='%s@testuser.valid.domain')
class MailBackendWhitelistBackendTest(TestCase):

    def test_whitify_mail_addresses_replace(self):
        email_1 = 'albertus.magnus@example.com'
        email_2 = 'thomas_von_aquin@example.com'
        processed_list = WhitelistEmailBackend.whitify_mail_addresses(mail_address_list=[email_1, email_2])

        self.assertEqual(len(processed_list), 2)
        self.assertEqual(processed_list[0], 'albertus.magnus_example.com@testuser.valid.domain')
        self.assertEqual(processed_list[1], 'thomas_von_aquin_example.com@testuser.valid.domain')

    def test_whitify_mail_addresses_whitelisted_domain(self):
        email = 'platon@valid.domain'
        processed_list = WhitelistEmailBackend.whitify_mail_addresses(mail_address_list=[email])

        self.assertEqual(len(processed_list), 1)
        self.assertEqual(processed_list[0], email)

    @override_settings(EMAIL_BACKEND_REDIRECT_ADDRESS='')
    def test_whitify_mail_addresses_no_redirect_configured(self):
        email = 'sokrates@example.com'
        processed_list = WhitelistEmailBackend.whitify_mail_addresses(mail_address_list=[email])

        self.assertEqual(len(processed_list), 0)

    def test_process_recipients_regular(self):
        mail = EmailMultiAlternatives('Test subject', 'Here is the message.', 'from@example.com', ['to@example.com'],
                                      connection=None)

        backend = WhitelistEmailBackend()
        message_list = backend._process_recipients([mail])
        self.assertEqual(len(message_list), 1)
        self.assertEqual(message_list[0].to, ['to_example.com@testuser.valid.domain'])



from unittest import mock

from django import forms
from django.contrib import admin
from django.contrib.auth.models import User
from django.test import TestCase

from ai_django_core.admin.model_admins.mixins import CommonInfoAdminMixin
from ai_django_core.tests.mixins import RequestProviderMixin
from testapp.models import CommonInfoBasedModel


class CommonInfoBasedModelForm(forms.ModelForm):
    class Meta:
        model = CommonInfoBasedModel
        exclude = ()


class TestCommonInfoAdminMixinAdmin(CommonInfoAdminMixin, admin.ModelAdmin):
    pass


class CommonInfoAdminMixinTest(RequestProviderMixin, TestCase):

    @classmethod
    def setUpTestData(cls):
        super().setUpTestData()

        cls.user = User.objects.create(username='my_user')
        cls.request = cls.get_request(cls.user)

        admin.site.register(CommonInfoBasedModel, TestCommonInfoAdminMixinAdmin)

    @classmethod
    def tearDownClass(cls):
        super().tearDownClass()

        admin.site.unregister(CommonInfoBasedModel)

    def test_readonly_fields_are_set(self):
        model_admin = TestCommonInfoAdminMixinAdmin(model=CommonInfoBasedModel, admin_site=admin.site)

        self.assertIn('created_by', model_admin.get_readonly_fields(self.request))
        self.assertIn('created_at', model_admin.get_readonly_fields(self.request))

        self.assertIn('lastmodified_by', model_admin.get_readonly_fields(self.request))
        self.assertIn('lastmodified_at', model_admin.get_readonly_fields(self.request))

    def test_created_by_is_set_on_creation(self):
        model_admin = TestCommonInfoAdminMixinAdmin(model=CommonInfoBasedModel, admin_site=admin.site)

        obj = model_admin.save_form(self.request, CommonInfoBasedModelForm(), False)

        self.assertEqual(obj.created_by, self.user)
        self.assertEqual(obj.lastmodified_by, self.user)

    def test_created_by_is_not_altered_on_update(self):
        model_admin = TestCommonInfoAdminMixinAdmin(model=CommonInfoBasedModel, admin_site=admin.site)

        other_user = User.objects.create(username='other_user')
        with mock.patch.object(CommonInfoBasedModel, 'get_current_user', return_value=other_user):
            obj = CommonInfoBasedModel.objects.create(value=1, created_by=other_user, lastmodified_by=other_user)

        form = CommonInfoBasedModelForm(instance=obj)
        obj = model_admin.save_form(self.request, form, True)

        self.assertEqual(obj.created_by, other_user)
        self.assertEqual(obj.lastmodified_by, self.user)
